﻿namespace HeuristicLab.Tests.Benchmark.Problem {
  using System;
  using System.Linq;
  using System.Threading.Tasks;

  using HeuristicLab.Algorithms.PushGP.Data.Random;
  using HeuristicLab.Algorithms.PushGP.Expressions;
  using HeuristicLab.Algorithms.PushGP.Generators;
  using HeuristicLab.Algorithms.PushGP.Interpreter;
  using HeuristicLab.BenchmarkSuite;

  using Microsoft.VisualStudio.TestTools.UnitTesting;

  [TestClass]
  public class CountOdds : Problem<long[], long> {
    public CountOdds() {
      //  this.Examples = this.GetExamples(
      //    "examples-count-odds.csv",
      //    ExampleConverter.Integers,
      //    ExampleConverter.Integer);
    }

    [TestMethod]
    [TestProperty("Time", "Medium")]
    [TestCategory("ProblemTest")]
    public void RandomWalk() {
      var maxProgramSizeLimit = 1024;
      var iterations = 4000;
      var best = double.MaxValue;
      var globalExecCounter = 0;
      var lockObj = new object();
      var lockCount = new object();
      RandomFactory.Seed = 1337;

      Expression bestProgram = null;
      var config = new Configuration { EvalPushLimit = 500 };
      var pool = new PushGpInterpreterPool(config);

      var descriptor = new BenchmarkSuite.Problems.CountOdds();
      var instance = new BenchmarkSuiteInstanceProvider();
      var examples = instance.LoadData(descriptor);

      Parallel.For(0, iterations, i => {
        var execCounter = 0;
        var program = CodeGenerator.RandomProgram(maxProgramSizeLimit);
        var results = new double[examples.TrainingCount];

        using (var interpreter = pool.GetInstance()) {
          for (var j = 0; j < examples.TrainingCount; j++) {
            examples.ApplyTrainingInput(interpreter, j);
            interpreter.Run(program);

            results[j] = examples.CompareTrainingOutput(interpreter, j);

            execCounter += interpreter.ExecCounter;
            interpreter.Clear();
          }
        }

        lock (lockCount) {
          globalExecCounter += execCounter;
        }

        var avg = results.Average();

        if (avg >= best) return;

        lock (lockObj) {
          if (avg < best) {
            best = avg;
            bestProgram = program;
          }
        }
      });

      var resultsTest = new double[examples.TrainingCount];
      Parallel.For(0, examples.TrainingCount, i => {
        using (var interpreter = pool.GetInstance()) {
          examples.ApplyTestInput(interpreter, i);
          interpreter.Run(bestProgram);

          resultsTest[i] = examples.CompareTestOutput(interpreter, i);
        }
      });

      var averageTestResult = resultsTest.Average();

      Console.WriteLine("Best Training: {0}", best);
      Console.WriteLine("Test: {0}", averageTestResult);
      Console.WriteLine("Best trainig program: {0}", bestProgram);
      Console.WriteLine("ExecCounter: {0}", globalExecCounter);
    }
  }
}